package org.codeproject.dao.test;

import static java.lang.System.out;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import java.util.List;

import org.apache.commons.lang3.RandomStringUtils;
import org.codeproject.dao.DaoRegistry;
import org.codeproject.dao.PostDao;
import org.codeproject.dao.PersistenceManager;
import org.codeproject.domain.Comment;
import org.codeproject.domain.Post;
import org.junit.Test;

public class TestPostDao {
	
	private PostDao postDao = (PostDao) DaoRegistry.instance().getDao(Post.class);
	
	@Test
	public void simpleSave() throws Exception {
		Post newPost = new Post();
		newPost.setTitle("simpleSave_" + RandomStringUtils.randomAscii(10));
		newPost.setContent("simpleSave_");
		
		PersistenceManager unitOfWork = new PersistenceManager();
		unitOfWork.add(newPost);
		unitOfWork.commit();
		
		assertNotNull("newPost has no id", newPost.getId());
	}
	
	private Post createPostWithComment() throws Exception {
		Post newPost = new Post();
		newPost.setTitle("saveWithComments_" + RandomStringUtils.randomAscii(10));
		newPost.setContent("saveWithComments_");
		
		Comment firstComment = new Comment();
		firstComment.setContent("Test-Comment1");
		newPost.getComments().add(firstComment);
		
		PersistenceManager unitOfWork = new PersistenceManager();
		unitOfWork.add(newPost);
		unitOfWork.commit();
		
		assertNotNull("newPost has no id", newPost.getId());
		
		return newPost;
	}
	
	@Test
	public void changePostAndComments() throws Exception {
		String updatedSuffix = "_updated";
		String addedCommentContent = "added_comment";
		
		Post createdPost = createPostWithComment();
		
		//change post title
		createdPost.setTitle(createdPost.getTitle().concat(updatedSuffix));
		
		//change first comment
		Comment existingComment = createdPost.getComments().iterator().next();
		existingComment.setContent(existingComment.getContent().concat(updatedSuffix));
		
		//add another comment
		Comment addedComment = new Comment();
		addedComment.setContent(addedCommentContent);
		createdPost.getComments().add(addedComment);
		
		//save all changes
		PersistenceManager unitOfWork = new PersistenceManager();
		unitOfWork.update(createdPost);
		unitOfWork.commit();
		
		//read back and check
		Post readBack = postDao.getById(createdPost.getId());
		assertTrue("wrong post title found", readBack.getTitle().endsWith(updatedSuffix));
		boolean foundAddedComment = false;
		boolean foundChangedComment = false;
		for (Comment comment : readBack.getComments()) {
			if (comment.getContent().equals(addedCommentContent)) {
				foundAddedComment = true;
			}
			if (comment.getContent().endsWith(updatedSuffix)) {
				foundChangedComment = true;
			}
		}
		assertTrue("added comment not found", foundAddedComment);
		assertTrue("changed comment not found", foundChangedComment);
	}
	
	@Test
	public void saveWithComment() throws Exception {
		createPostWithComment();
	}
	
	@Test
	public void saveAndDeleteWithComment() throws Exception {
		Post addedPost = createPostWithComment();
		
		PersistenceManager unitOfWork = new PersistenceManager();
		unitOfWork.delete(addedPost);
		unitOfWork.commit();
		
		assertNull("post not deleted", postDao.getById(addedPost.getId()));
	}
	
	@Test
	public void saveTwo() throws Exception {
		Post firstPost = new Post();
		String firstPostTitle = "saveTwo-first-" + RandomStringUtils.randomAscii(10);
		firstPost.setTitle(firstPostTitle);
		
		Post secondPost = new Post();
		String secondPostTitle = "saveTwo-second-" + RandomStringUtils.randomAscii(10);
		secondPost.setTitle(secondPostTitle);
		
		PersistenceManager unitOfWork = new PersistenceManager();
		unitOfWork.add(firstPost);
		unitOfWork.add(secondPost);
		unitOfWork.commit();
		
		assertNotNull("firstPost has no id", firstPost.getId());
		assertNotNull("secondPost has no id", secondPost.getId());
	}
	
	private Post createRandomPost() throws Exception {
		Post newPost = new Post();
		String title = "getById_" + RandomStringUtils.randomAscii(10);
		newPost.setTitle(title);
		newPost.setContent("getById_");
		
		PersistenceManager unitOfWork = new PersistenceManager();
		unitOfWork.add(newPost);
		unitOfWork.commit();
		
		return newPost;
	}
	
	@Test
	public void getById() throws Exception {
		Post createdPost = createRandomPost();
		
		Post readBackPost = postDao.getById(createdPost.getId());
		
		assertNotNull("post not found", readBackPost);
		assertEquals("created post title not equals \"read back\" post title", readBackPost.getTitle(), createdPost.getTitle());
	}
	
	@Test
	public void getAll() throws Exception {
		createRandomPost();
		
		List<Post> posts = postDao.getAll();
		
		assertNotNull(posts);
		
		for(Post post : posts) {
			out.println(String.format("%s - %s", post.getTitle(), post.getCreationDate()));
		}
		
		assertTrue("cannot find any post in database", posts.size() > 0);
	}
	
}
