#include "json-util.h"

#define DIR_ACCESS_MODE 0777


static void bson2json_rec (bson_iterator *i, JsonBuilder *builder, gboolean in_obj);
static void add_node (const gchar* name, JsonNode *node, bson *b);


static void foreach_member (JsonObject *object, const char *name, JsonNode *node, gpointer data) {
  add_node (name, node, (bson *) data);
}

static void foreach_element (JsonArray *array, guint index, JsonNode *node, gpointer data) {
  gchar *name;
  name = g_strdup_printf ("%d", index);
  add_node (name, node, (bson *) data);
  g_free (name);
}

static void add_node (const gchar* name, JsonNode *node, bson *b) {
  bson_oid_t oid[1];
  
  switch (JSON_NODE_TYPE (node)) {
    case JSON_NODE_OBJECT:
      bson_append_start_object (b, name);

      json_object_foreach_member (json_node_get_object (node), foreach_member, b);

      bson_append_finish_object (b);
      break;

    case JSON_NODE_ARRAY:
      bson_append_start_array (b, name);

      json_array_foreach_element (json_node_get_array (node), foreach_element, b);

      bson_append_finish_array (b);
      break;

    case JSON_NODE_NULL:
      bson_append_null (b, name);
      break;

    case JSON_NODE_VALUE:
      switch (json_node_get_value_type (node)) {
        case G_TYPE_INT64:
          bson_append_long (b, name, json_node_get_int (node));
          break;
        case G_TYPE_DOUBLE:
          bson_append_double (b, name, json_node_get_double (node));
          break;
        case G_TYPE_BOOLEAN:
          bson_append_bool
            (b, name, (json_node_get_boolean (node) == TRUE)? 1 : 0);
          break;
        case G_TYPE_STRING:
          if (g_strcmp0 (name, "_id"))
            bson_append_string (b, name, json_node_get_string (node));
          else {
            bson_oid_from_string (oid, json_node_get_string (node));
            bson_append_oid (b, name, oid);
          }
          break;
      }
      break;
  }
}


static void bson2json_rec (bson_iterator *i, JsonBuilder *builder, gboolean in_obj) {
  const gchar *key;
  bson_iterator sub[1];
  char str[25];
  
  while (bson_iterator_next (i)) {
    bson_type type = bson_iterator_type (i);
    if (type == BSON_EOO)
      break;
    key = bson_iterator_key (i);

    if (in_obj) {
      switch (type) {
        case BSON_OBJECT:
        case BSON_ARRAY:
        case BSON_STRING:
        case BSON_DOUBLE:
        case BSON_INT:
        case BSON_LONG:
        case BSON_NULL:
        case BSON_BOOL:
        case BSON_OID:
          json_builder_set_member_name (builder, key);
        default:
          ;
      }
    }
    
    switch (type) {

      case BSON_OBJECT:
        json_builder_begin_object (builder);

        bson_iterator_subiterator (i, sub);
        bson2json_rec (sub, builder, TRUE);
        
        json_builder_end_object (builder);
          
        break;
      case BSON_ARRAY:
        json_builder_begin_array (builder);

        bson_iterator_subiterator (i, sub);
        bson2json_rec (sub, builder, FALSE);        

        json_builder_end_array (builder);
        break;

      case BSON_STRING:
        json_builder_add_string_value (builder, bson_iterator_string (i));
        break;

      case BSON_DOUBLE:
        json_builder_add_double_value (builder, bson_iterator_double (i));
        break;

      case BSON_INT:
      case BSON_LONG:
        json_builder_add_int_value (builder, bson_iterator_int (i));
        break;

      case BSON_NULL:
        json_builder_add_null_value (builder);
        break;

      case BSON_BOOL:
        json_builder_add_boolean_value (builder, (bson_iterator_bool (i) != 0)? TRUE : FALSE);
        break;

      case BSON_OID:
        
        bson_oid_to_string (bson_iterator_oid (i), str);
        json_builder_add_string_value (builder, str);
        break;

      case BSON_DATE:
      case BSON_BINDATA:
      case BSON_UNDEFINED:
      case BSON_REGEX:
      case BSON_CODE:
      case BSON_CODEWSCOPE:
      case BSON_TIMESTAMP:
      default:
        g_warning ("unknown bson-type: %d\n", type);
    }
  }
}

/**
 * bson2json:
 * @bson: a bson document
 *
 * Convert a BSON document to a JSON text
 *
 * Return value: The #JsonNode or %NULL if unsuccessfull.
 */
JsonNode *bson2json (bson *b) {
  JsonBuilder *builder;
  JsonNode *node;
  bson_iterator i[1];

  builder = json_builder_new ();
  bson_iterator_init (i, b);

  json_builder_begin_object (builder);
  bson2json_rec (i, builder, TRUE);
  json_builder_end_object (builder);
  
  node = json_builder_get_root (builder);
  g_object_unref (builder);
  return node;
}


/**
 * json2bson:
 * @node: a json text
 *
 * Convert a JSON text to a BSON document
 *
 * Return value: The #JsonNode or %NULL if unsuccessfull.
 */
bson *json2bson (JsonNode *node) {
  bson *b;

  if (JSON_NODE_TYPE (node) != JSON_NODE_OBJECT)
    ;

  b = g_new (bson, 1);
  bson_init (b);

  json_object_foreach_member (json_node_get_object (node), foreach_member, b);

  if (bson_finish (b) != BSON_OK)
    ;

  return b;
}


gchar *json2string (JsonNode *node) {
  JsonGenerator *generator = json_generator_new ();
  gchar *string;

  json_generator_set_root (generator, node);

  g_object_set (generator,
                "pretty", TRUE,
                "indent", 2,
                "indent-char", ' ', NULL);

  string = json_generator_to_data (generator, NULL);
  
  g_object_unref (generator);
  return string;
}


JsonNode *string2json (const gchar *data) {
  JsonParser *parser;
  JsonNode *node;
  GError *error;

  parser = json_parser_new ();
  
  error = NULL;
  if (!json_parser_load_from_data (parser, data, -1, &error)) {
    g_warning ("%s\n", error->message );
    g_error_free (error);
    g_object_unref (parser);
    return NULL;
  }

  node = json_node_copy (json_parser_get_root (parser));
  g_object_unref (parser);
  return node;
}


void json_print (JsonNode *node) {
  gchar *string;
  string = json2string (node);
  g_print ("%s\n", string);
  g_free (string);
}



JsonNode *json_read (const gchar *filename) {
  GError *error;
  gchar *data;
  JsonNode *node;

  error = NULL;  
  if (!g_file_get_contents (filename, &data, NULL, &error)) {
    g_warning ("%s\n", error->message);
    g_error_free (error);
    return NULL;
  }

  g_assert (data != NULL);
  node = string2json (data);

  return node;
}


gboolean json_write (JsonNode *node, const gchar *filename) {
  GError *error;
  gchar *dir;
  gchar *data;

  dir = g_path_get_dirname (filename);
  g_mkdir_with_parents (dir, DIR_ACCESS_MODE);
  g_free (dir);

  data = json2string (node);
  g_assert (data != NULL);

  error = NULL;
  if (! g_file_set_contents (filename, data, -1, &error)) {
    g_warning ("%s\n", error->message);
    g_error_free (error);
    g_free (data);
    return FALSE;
  }

  g_free (data);
  return TRUE;
}
