#include <glib.h>
#include "shs-node.h"


ShsNode *node_new (gpointer data) {
  ShsNode *node = g_new (ShsNode, 1);

  node->parent = node->children = node->next = node->prev = NULL;
  node->data = data;
  return node;
}

ShsNode *node_parent (ShsNode *node) {
  return (node != NULL)? node->parent : NULL;
}

ShsNode *node_first_child (ShsNode *node) {
  return (node != NULL)? node->children : NULL;
}

ShsNode *node_last_child (ShsNode *node) {
  return (node != NULL)? node_last_sibling (node->children) : NULL;
}

ShsNode *node_next_sibling (ShsNode *node) {
  return (node != NULL
          && node_parent (node->next) == node->parent)?
    node->next : NULL;
}

ShsNode *node_prev_sibling (ShsNode *node) {
  return (node != NULL
          && node_parent (node->prev) == node->parent)?
    node->prev : NULL;
}

ShsNode *node_first_sibling (ShsNode *node) {
  return (node != NULL)? node->parent->children : NULL;
}

ShsNode *node_last_sibling (ShsNode *node) {
  ShsNode *n;
  if (node == NULL)
    return NULL;

  do { n = node; }
  while ((node = node_next_sibling (node)) != NULL);
    
  return n;
}

ShsNode *node_next (ShsNode *node) {
  return node->next;
}

ShsNode *node_prev (ShsNode *node) {
  return node->prev;
}


void node_traverse_pre_order (ShsNode *node, ShsNodeTraverseFunc func, gpointer data) {
  if (node == NULL)
    return;

  func (node, data);
  node_traverse_pre_order (node_first_child (node), func, data);
  node_traverse_pre_order (node_next_sibling (node), func, data);
}


/* --- */
guint node_depth (ShsNode *node) {
  guint depth;
  depth = 0;

  while (node != NULL) {
    node = node->parent;
    ++depth;
  }
  return depth;
}

void node_append (ShsNode *root, ShsNode *node) {
  ShsNode *n;

  if (root == NULL || node == NULL)
    return;

  node->parent = root;
  node->prev = NULL;
  node->next = NULL;
  n = root;

  while (n->children == NULL && n->prev != NULL)
    n = n->prev;

  n = node_last_sibling (n->children);
  node->prev = n;

  if (n != NULL) {
    if (n->next != NULL) {
      n->next->prev = n;
      node->next = n->next;
    }
    n->next = node;
  }

  if (root->children == NULL)
    root->children = node;
}




#ifdef SHS_TEST

static void node_print_label (ShsNode *node, gpointer data) {
  int i;
  for (i=0; i< node_depth (node); ++i)
    g_print ("  ");
  g_print ("%s\n", (gchar *) node->data);
}


static void node_print (ShsNode *n) {
  node_traverse_pre_order (n, node_print_label, NULL);
}

static void node_test () {
/*  A
 *  +- B
 *  |  +- C
 *  |  +- D
 *  +- E
 *     +- F
 *     +- G
 */

  ShsNode *a;
  ShsNode *b;
  ShsNode *c;
  ShsNode *d;
  ShsNode *e;
  ShsNode *f;
  ShsNode *g;

  a = node_new (g_strdup ("a"));
  b = node_new (g_strdup ("b"));
  c = node_new (g_strdup ("c"));
  d = node_new (g_strdup ("d"));
  e = node_new (g_strdup ("e"));
  f = node_new (g_strdup ("f"));
  g = node_new (g_strdup ("g"));

  node_append (a, b);
  node_append (b, c);
  node_append (b, d);
  node_append (a, e);
  node_append (e, f);
  node_append (e, g);

  node_print (a);
  
  g_free (a->data); g_free (a);
  g_free (b->data); g_free (b);
  g_free (c->data); g_free (c);
  g_free (d->data); g_free (d);
  g_free (e->data); g_free (e);
  g_free (f->data); g_free (f);
  g_free (g->data); g_free (g);
}


int main (int argc, char *argv[]) {
  node_test ();

  return 0;
}

#endif /* SHS_TEST */

