#include <gtk/gtk.h>
#include <assert.h>

/* GLOBAL VARIABLES */
static gchar *filename = NULL;

static GtkWidget *text_view;
static GtkTextBuffer *buffer;
static GtkTextTagTable *tag_table;

static GObject *tool_button_bold;
static GObject *tool_button_italic;
static GObject *tool_button_underline;

static gboolean update_state = FALSE;

/* TAGS */
void create_tags (void)
{
  gtk_text_buffer_create_tag (buffer, "bold", 
			      "weight", PANGO_WEIGHT_BOLD, 
			      NULL);
  gtk_text_buffer_create_tag (buffer, "italic", 
			      "style", PANGO_STYLE_ITALIC, 
			      NULL);
  gtk_text_buffer_create_tag (buffer, "underline", 
			      "underline", PANGO_UNDERLINE_SINGLE, 
			      NULL);
}

void apply_tag (const char *name)
{
  GtkTextIter start, end;
  gtk_text_buffer_get_selection_bounds (buffer, &start, &end);
  
  gtk_text_buffer_apply_tag_by_name (buffer, name, &start, &end);
}

void remove_tag (const char *name)
{
  GtkTextIter start, end;
  gtk_text_buffer_get_selection_bounds (buffer, &start, &end);

  gtk_text_buffer_remove_tag_by_name (buffer, name, &start, &end);
}


void toggle (GtkToggleButton *button, const char *name)
{
  if (update_state)
    return;

  gboolean active;
  g_object_get (button, "active", &active, NULL);
  if (active)
    apply_tag (name);
  else
    remove_tag (name);
}

gboolean has_tag (GtkTextIter *iter, char *name)
{
  GtkTextTag *tag = gtk_text_tag_table_lookup (tag_table, name);

  return (gtk_text_iter_has_tag (iter, tag));
}

void update (GtkTextBuffer *buffer, GtkTextIter *iter, GtkTextMark *mark, gpointer data)
{
  update_state = TRUE;
  g_object_set (tool_button_bold, "active", 
		has_tag (iter, "bold"),
		NULL);
  g_object_set (tool_button_italic, "active", 
		has_tag (iter, "italic"),
		NULL);
  g_object_set (tool_button_underline, "active", 
		has_tag (iter, "underline"),
		NULL);
  update_state = FALSE;
}

/* FILE */
void file_new (GtkMenuItem *item, gpointer data)
{
  g_free (filename);
  filename = NULL;
  gtk_text_buffer_set_text (buffer, "", -1);
}

void file_open (GtkMenuItem *item, gpointer data)
{
  GtkWidget *dialog;
  dialog = gtk_file_chooser_dialog_new ("Open File", NULL, GTK_FILE_CHOOSER_ACTION_OPEN,
					GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
					GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, 
					NULL);

  if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
    {
      gchar *text;

      g_free (filename);
      filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
      
      g_file_get_contents (filename, &text, NULL, NULL);
      g_object_set (buffer, "text", text, NULL);

      g_free (text);
    }
  gtk_widget_destroy (dialog);
}

void file_save_as (GtkMenuItem *item, gpointer data)
{
  GtkWidget *dialog;
  dialog = gtk_file_chooser_dialog_new ("Save File", NULL, GTK_FILE_CHOOSER_ACTION_SAVE,
					GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
					GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, 
					NULL);

  if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
    {
      g_free (filename);
      filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
    }
  gtk_widget_destroy (dialog);
}

void file_save (GtkMenuItem *item, gpointer data)
{
  if (filename == NULL)
    file_save_as (item, data);
  
  if (filename != NULL)
    {
      gchar *text;

      g_object_get (buffer, "text", &text, NULL);
      g_file_set_contents (filename, text, -1, NULL);
  
      g_free (text);
    }
}


/* COPY AND PASTE */
void cut (GtkMenuItem *item, gpointer data)
{
  g_signal_emit_by_name (text_view, "cut-clipboard", NULL);
}

void copy (GtkMenuItem *item, gpointer data)
{
  g_signal_emit_by_name (text_view, "copy-clipboard", NULL);
}

void paste (GtkMenuItem *item, gpointer data)
{
  g_signal_emit_by_name (text_view, "paste-clipboard", NULL);
}


int main (int argc, char *argv[])
{
  gtk_init (&argc, &argv);

  GtkBuilder *builder = gtk_builder_new ();
  gtk_builder_add_from_file (builder, "ui.xml", NULL);
  gtk_builder_connect_signals (builder, NULL);

  GtkWidget *window = GTK_WIDGET (gtk_builder_get_object (builder, "window"));
  assert (window != NULL);


  text_view = GTK_WIDGET  (gtk_builder_get_object (builder, "text_view"));
  g_object_get (text_view, "buffer", &buffer, NULL);
  assert (text_view != NULL && buffer != NULL);
  g_signal_connect (buffer, "mark-set", G_CALLBACK (update), NULL);
  g_object_get (buffer, "tag-table", &tag_table, NULL);
  assert (tag_table);

  tool_button_bold = gtk_builder_get_object (builder, "toolbutton_bold");
  tool_button_italic = gtk_builder_get_object (builder, "toolbutton_italic");
  tool_button_underline = gtk_builder_get_object (builder, "toolbutton_underline");
  assert (tool_button_bold != NULL 
	  && tool_button_italic != NULL
	  && tool_button_underline != NULL);

  g_signal_connect (tool_button_bold, "clicked", G_CALLBACK (toggle), "bold");
  g_signal_connect (tool_button_italic, "toggled", G_CALLBACK (toggle), "italic");
  g_signal_connect (tool_button_underline, "toggled", G_CALLBACK (toggle), "underline");

  
  gtk_text_buffer_set_text (buffer, "Hello world.\n", -1);
  create_tags ();

  g_object_unref (builder);  
  gtk_widget_show_all (window);  
  gtk_main ();

  return 0;
}
